@@ -36,6 +36,8 @@ extern "C" {
 // https://bugs.gentoo.org/show_bug.cgi?id=522908
 #define WERROR ("-W" "error")
 
+#define PATH_ALLOWED_CHARS "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789+,-./_"
+
 using namespace std;
 
 /* Adjust and run make_cmd to build a kernel module. */
@@ -132,7 +134,8 @@ make_any_make_cmd(systemtap_session& s, const string& dir, const string& target)
 static vector<string>
 make_make_cmd(systemtap_session& s, const string& dir)
 {
-  return make_any_make_cmd(s, dir, "modules");
+  vector<string> mc = make_any_make_cmd(s, dir, "modules");
+  return mc;
 }
 
 static vector<string>
@@ -362,6 +365,8 @@ compile_pass (systemtap_session& s)
                   "STAPCONF_KERNEL_STACKTRACE", NULL);
   output_autoconf(s, o, "autoconf-save-stack-trace-no-bp.c",
                   "STAPCONF_KERNEL_STACKTRACE_NO_BP", NULL);
+  output_autoconf(s, o, "autoconf-unwind-stack-trace.c",
+                  "STAPCONF_KERNEL_UNWIND_STACK", NULL);
   output_autoconf(s, o, "autoconf-asm-syscall.c",
 		  "STAPCONF_ASM_SYSCALL_H", NULL);
   output_autoconf(s, o, "autoconf-ring_buffer-flags.c", "STAPCONF_RING_BUFFER_FLAGS", NULL);
@@ -404,8 +409,8 @@ compile_pass (systemtap_session& s)
   output_autoconf(s, o, "autoconf-smpcall-4args.c", "STAPCONF_SMPCALL_4ARGS", NULL);
 
   // used by tapset/timestamp_monotonic.stp
-  output_exportconf(s, o, "cpu_clock", "STAPCONF_CPU_CLOCK");
-  output_exportconf(s, o, "local_clock", "STAPCONF_LOCAL_CLOCK");
+  output_autoconf(s, o, "autoconf-cpu-clock.c", "STAPCONF_CPU_CLOCK", NULL);
+  output_autoconf(s, o, "autoconf-local-clock.c", "STAPCONF_LOCAL_CLOCK", NULL);
 
   // used by runtime/uprobe-inode.c
   output_either_exportconf(s, o, "uprobe_register", "register_uprobe",
@@ -438,7 +443,9 @@ compile_pass (systemtap_session& s)
   output_exportconf(s, o, "vmalloc_node", "STAPCONF_VMALLOC_NODE");
 
   // RHBZ1233912 - s390 temporary workaround for non-atomic udelay()
-  output_exportconf(s, o, "udelay_simple", "STAPCONF_UDELAY_SIMPLE");
+  output_exportconf(s, o, "udelay_simple", "STAPCONF_UDELAY_SIMPLE_EXPORTED");
+  output_autoconf(s, o, "autoconf-udelay_simple.c", "STAPCONF_UDELAY_SIMPLE",
+		  NULL);
 
   output_autoconf(s, o, "autoconf-tracepoint-strings.c", "STAPCONF_TRACEPOINT_STRINGS", NULL);
   output_autoconf(s, o, "autoconf-timerfd.c", "STAPCONF_TIMERFD_H", NULL);
@@ -452,6 +459,15 @@ compile_pass (systemtap_session& s)
 		  "STAPCONF_GET_USER_PAGES_REMOTE_FLAGS", NULL);
   output_autoconf(s, o, "autoconf-get_user_pages_remote-flags_locked.c",
 		  "STAPCONF_GET_USER_PAGES_REMOTE_FLAGS_LOCKED", NULL);
+  output_autoconf(s, o, "autoconf-bio-bi_opf.c", "STAPCONF_BIO_BI_OPF", NULL);
+  output_autoconf(s, o, "autoconf-linux-sched_headers.c",
+		  "STAPCONF_LINUX_SCHED_HEADERS", NULL);
+
+  // used by runtime/linux/netfilter.c
+  output_exportconf(s, o, "nf_register_hook", "STAPCONF_NF_REGISTER_HOOK");
+
+  // used by tapset/linux/ioblock.stp
+  output_exportconf(s, o, "disk_get_part", "STAPCONF_DISK_GET_PART");
 
   o << module_cflags << " += -include $(STAPCONF_HEADER)" << endl;
 
@@ -495,7 +511,18 @@ compile_pass (systemtap_session& s)
   #if CHECK_POINTER_ARITH_PR5947
   o << "EXTRA_CFLAGS += -Wpointer-arith" << endl;
   #endif
-  o << "EXTRA_CFLAGS += -I\"" << s.runtime_path << "\"" << endl;
+
+  // If we've got a reasonable runtime path from the user, we'll just
+  // do '-IDIR'. If there are any sneaky/odd characters in it, we'll
+  // have to quote it, like '-I"DIR"'.
+  if (s.runtime_path.find_first_not_of(PATH_ALLOWED_CHARS, 0) == string::npos)
+    o << "EXTRA_CFLAGS += -I" << s.runtime_path << endl;
+  else
+    {
+      s.print_warning("quoting runtime path in the module Makefile.");
+      o << "EXTRA_CFLAGS += -I\"" << s.runtime_path << "\"" << endl;
+    }
+
   // XXX: this may help ppc toc overflow
   // o << "CFLAGS := $(subst -Os,-O2,$(CFLAGS)) -fminimal-toc" << endl;
   o << "obj-m := " << s.module_name << ".o" << endl;
@@ -542,7 +569,6 @@ compile_pass (systemtap_session& s)
   for (unsigned i=0; i<s.auxiliary_outputs.size(); i++)
     o << s.auxiliary_outputs[i]->filename << ": $(STAPCONF_HEADER)" << endl;  
 
-
   o.close ();
 
   // Generate module directory pathname and make sure it exists.
@@ -560,6 +586,12 @@ compile_pass (systemtap_session& s)
 
   // Run make
   vector<string> make_cmd = make_make_cmd(s, s.tmpdir);
+  if (s.keep_tmpdir)
+    {
+      string E_source = s.translated_source.substr(s.translated_source.find_last_of("/")+1);
+      E_source.at(E_source.length() - 1) = 'i'; // overwrite the last character
+      make_cmd.push_back(E_source);
+    }
   rc = run_make_cmd(s, make_cmd);
   if (rc)
     s.set_try_server ();
@@ -714,8 +746,7 @@ uprobes_pass (systemtap_session& s)
   return rc;
 }
 
-static
-vector<string>
+static vector<string>
 make_dyninst_run_command (systemtap_session& s, const string& remotedir,
 			  const string&)
 {
@@ -752,13 +783,10 @@ make_dyninst_run_command (systemtap_session& s, const string& remotedir,
   return cmd;
 }
 
-vector<string>
-make_run_command (systemtap_session& s, const string& remotedir,
-                  const string& version)
+static vector<string>
+make_kernel_run_command (systemtap_session& s, const string& remotedir,
+			 const string& version)
 {
-  if (s.runtime_usermode_p())
-    return make_dyninst_run_command(s, remotedir, version);
-
   // for now, just spawn staprun
   vector<string> cmd { getenv("SYSTEMTAP_STAPRUN") ?: BINDIR "/staprun" };
 
@@ -784,6 +812,9 @@ make_run_command (systemtap_session& s, const string& remotedir,
   if (s.buffer_size)
     cmd.insert(cmd.end(), { "-b", lex_cast(s.buffer_size) });
 
+  if (s.read_stdin)
+    cmd.insert(cmd.end(), "-i");
+
   if (s.need_uprobes && !kernel_built_uprobes(s))
     {
       string opt_u = "-u";
@@ -828,6 +859,50 @@ make_run_command (systemtap_session& s, const string& remotedir,
   return cmd;
 }
 
+static vector<string>
+make_bpf_run_command (systemtap_session& s, const string& remotedir,
+		      const string&)
+{
+  vector<string> cmd;
+  cmd.push_back(getenv("SYSTEMTAP_STAPBPF") ?: BINDIR "/stapbpf");
+
+  for (unsigned i=1; i<s.verbose; i++)
+    cmd.push_back("-v");
+  if (s.suppress_warnings)
+    cmd.push_back("-w");
+
+  if (!s.output_file.empty())
+    {
+      cmd.push_back("-o");
+      cmd.push_back(s.output_file);
+    }
+
+  cmd.push_back((remotedir.empty() ? s.tmpdir : remotedir)
+		+ "/" + s.module_filename());
+
+  // FIXME add module arguments
+  // ??? Maybe insert these via BEGIN probe.
+  // cmd.insert(cmd.end(), s.globalopts.begin(), s.globalopts.end());
+
+  return cmd;
+}
+
+vector<string>
+make_run_command (systemtap_session& s, const string& remotedir,
+                  const string& version)
+{
+  switch (s.runtime_mode)
+    {
+    case systemtap_session::kernel_runtime:
+      return make_kernel_run_command (s, remotedir, version);
+    case systemtap_session::dyninst_runtime:
+      return make_dyninst_run_command(s, remotedir, version);
+    case systemtap_session::bpf_runtime:
+      return make_bpf_run_command(s, remotedir, version);
+    default:
+      abort();
+    }
+}
 
 // Build tiny kernel modules to query tracepoints.
 // Given a (header-file -> test-contents) map, compile them ASAP, and return
