@@ -26,6 +26,7 @@ struct option stap_long_options[] = {
   { "list-servers",                optional_argument, NULL, LONG_OPT_LIST_SERVERS },
   { "trust-servers",               optional_argument, NULL, LONG_OPT_TRUST_SERVERS },
   { "use-server-on-error",         optional_argument, NULL, LONG_OPT_USE_SERVER_ON_ERROR },
+  { "use-http-server",             required_argument, NULL, LONG_OPT_USE_HTTP_SERVER },
   { "all-modules",                 no_argument,       NULL, LONG_OPT_ALL_MODULES },
   { "remote",                      required_argument, NULL, LONG_OPT_REMOTE },
   { "remote-prefix",               no_argument,       NULL, LONG_OPT_REMOTE_PREFIX },
@@ -49,6 +50,7 @@ struct option stap_long_options[] = {
   { "suppress-time-limits",        no_argument,       NULL, LONG_OPT_SUPPRESS_TIME_LIMITS },
   { "runtime",                     required_argument, NULL, LONG_OPT_RUNTIME },
   { "dyninst",                     no_argument,       NULL, LONG_OPT_RUNTIME_DYNINST },
+  { "bpf",                         no_argument,       NULL, LONG_OPT_RUNTIME_BPF },
   { "benchmark-sdt",               no_argument,       NULL, LONG_OPT_BENCHMARK_SDT },
   { "benchmark-sdt-loops",         required_argument, NULL, LONG_OPT_BENCHMARK_SDT_LOOPS },
   { "benchmark-sdt-threads",       required_argument, NULL, LONG_OPT_BENCHMARK_SDT_THREADS },
@@ -59,5 +61,6 @@ struct option stap_long_options[] = {
   { "target-namespaces",           required_argument, NULL, LONG_OPT_TARGET_NAMESPACES },
   { "monitor",                     optional_argument, NULL, LONG_OPT_MONITOR },
   { "interactive",                 no_argument,       NULL, LONG_OPT_INTERACTIVE},
+  { "example",                     no_argument,       NULL, LONG_OPT_RUN_EXAMPLE},
   { NULL, 0, NULL, 0 }
 };
