@@ -1,5 +1,5 @@
 // systemtap translator/driver
-// Copyright (C) 2005-2016 Red Hat Inc.
+// Copyright (C) 2005-2018 Red Hat Inc.
 // Copyright (C) 2005 IBM Corp.
 // Copyright (C) 2006 Intel Corporation.
 //
@@ -22,12 +22,14 @@
 #include "rpm_finder.h"
 #include "task_finder.h"
 #include "csclient.h"
+#include "client-nss.h"
 #include "remote.h"
 #include "tapsets.h"
 #include "setupdwfl.h"
 #ifdef HAVE_LIBREADLINE
 #include "interactive.h"
 #endif
+#include "bpf.h"
 
 #if ENABLE_NLS
 #include <libintl.h>
@@ -400,6 +402,14 @@ static int collect_stpm(const char* fpath, const struct stat*,
   return FTW_CONTINUE;
 }
 
+#if !HAVE_BPF_DECLS
+int
+translate_bpf_pass (systemtap_session &)
+{
+  return 1;
+}
+#endif
+
 // Compilation passes 0 through 4
 int
 passes_0_4 (systemtap_session &s)
@@ -420,11 +430,11 @@ passes_0_4 (systemtap_session &s)
   // Perform passes 0 through 4 using a compile server?
   if (! s.specified_servers.empty ())
     {
-#if HAVE_NSS
+#if NEED_BASE_CLIENT_CODE
       compile_server_client client (s);
       return client.passes_0_4 ();
 #else
-      s.print_warning(_("Without NSS, using a compile-server is not supported by this version of systemtap"));
+      s.print_warning(_("Without NSS or HTTP client support, using a compile-server is not supported by this version of systemtap"));
 
       // This cannot be an attempt to use a server after a local compile failed
       // since --use-server-on-error is locked to 'no' if we don't have
@@ -455,16 +465,17 @@ passes_0_4 (systemtap_session &s)
     }
 
   // Now that no further changes to s.kernel_build_tree can occur, let's use it.
-  if (s.runtime_mode == systemtap_session::kernel_runtime) {
-    if ((rc = s.parse_kernel_config ()) != 0
-        || (rc = s.parse_kernel_exports ()) != 0
-        || (rc = s.parse_kernel_functions ()) != 0)
-      {
-        // Try again with a server
-        s.set_try_server ();
-        return rc;
-      }
-  }
+  if (!s.runtime_usermode_p())
+    {
+      if ((rc = s.parse_kernel_config ()) != 0
+	  || (rc = s.parse_kernel_exports ()) != 0
+	  || (rc = s.parse_kernel_functions ()) != 0)
+	{
+	  // Try again with a server
+	  s.set_try_server ();
+	  return rc;
+	}
+    }
 
   // Create the name of the C source file within the temporary
   // directory.  Note the _src prefix, explained in
@@ -500,12 +511,43 @@ passes_0_4 (systemtap_session &s)
 	}
       else if (s.script_file != "")
         {
+          if (s.run_example)
+            {
+              files.clear();
+              path_dir = string(PKGDATADIR) + "/examples";
+              (void) nftw(path_dir.c_str(), collect_stp, 1, FTW_ACTIONRETVAL);
+
+              vector<string> examples;
+              for (auto it = files.begin(); it != files.end(); ++it)
+                {
+                  string::size_type last_slash_index = it->find_last_of('/');
+                  string example_name = it->substr(last_slash_index + 1);
+                  if (s.script_file == example_name)
+                    examples.push_back(*it);
+                }
+
+              if (examples.size() > 1)
+                {
+                  cerr << "Multiple examples found: " << endl;
+                  for (auto it = examples.begin(); it != examples.end(); ++it)
+                    cerr << "  " << *it << endl;
+                  return 1;
+                }
+              else if (examples.size() == 0)
+                {
+                  cerr << _F("Example '%s' was not found under '%s'", s.script_file.c_str(), path_dir.c_str()) << endl;
+                  return 1;
+                }
+              else
+                  s.script_file = examples[0];
+            }
+
 	  user_file_stat_rc = stat (s.script_file.c_str(), & user_file_stat);
 	}
       // otherwise, rc is 0 for a command line script
 
       vector<string> version_suffixes;
-      if (s.runtime_mode == systemtap_session::kernel_runtime)
+      if (!s.runtime_usermode_p())
         {
 	  // Construct kernel-versioning search path
 	  string kvr = s.kernel_release;
@@ -546,6 +588,8 @@ passes_0_4 (systemtap_session &s)
 	runtime_prefix = "/linux";
       else if (s.runtime_mode == systemtap_session::dyninst_runtime)
 	runtime_prefix = "/dyninst";
+      else if (s.runtime_mode == systemtap_session::bpf_runtime)
+        runtime_prefix = "/bpf";
       if (!runtime_prefix.empty())
 	for (unsigned i=0; i<version_suffixes.size(); i+=2)
 	    version_suffixes.insert(version_suffixes.begin() + i/2,
@@ -764,7 +808,10 @@ passes_0_4 (systemtap_session &s)
       user_flags |= pf_user_file;
       if (s.script_file == "-")
         {
-          s.user_files.push_back (parse (s, "<input>", cin, user_flags));
+	  if (s.stdin_script.str().empty())
+	    s.stdin_script << cin.rdbuf();	    
+          s.user_files.push_back (parse (s, "<input>", s.stdin_script,
+					 user_flags));
         }
       else if (s.script_file != "")
         {
@@ -897,6 +944,23 @@ passes_0_4 (systemtap_session &s)
   PROBE1(stap, pass2__start, &s);
   rc = semantic_pass (s);
 
+  // http handled probes need probe information from pass 2
+  if (! s.http_servers.empty ())
+    {
+#if NEED_BASE_CLIENT_CODE
+      compile_server_client client (s);
+      return client.passes_0_4 ();
+#else
+      s.print_warning(_("Without NSS or HTTP client support, using a compile-server is not supported by this version of systemtap"));
+
+      // This cannot be an attempt to use a server after a local compile failed
+      // since --use-server-on-error is locked to 'no' if we don't have
+      // NSS.
+      assert (! s.try_server ());
+      s.print_warning(_("Ignoring --use-server"));
+#endif
+    }
+
   // Dump a list of known probe point types, if requested.
   if (s.dump_mode == systemtap_session::dump_probe_types)
     s.pattern_root->dump (s);
@@ -1014,50 +1078,62 @@ passes_0_4 (systemtap_session &s)
     }
 
   // PASS 3: TRANSLATION
-  s.verbose = s.perpass_verbose[2];
-  times (& tms_before);
-  gettimeofday (&tv_before, NULL);
-  PROBE1(stap, pass3__start, &s);
-
-  rc = translate_pass (s);
-  if (! rc && s.last_pass == 3)
+  // (BPF does translation and compilation at once in pass 4.)
+  if (s.runtime_mode != systemtap_session::bpf_runtime)
     {
-      ifstream i (s.translated_source.c_str());
-      cout << i.rdbuf();
+      s.verbose = s.perpass_verbose[2];
+      times (& tms_before);
+      gettimeofday (&tv_before, NULL);
+      PROBE1(stap, pass3__start, &s);
+
+      rc = translate_pass (s);
+
+      if (! rc && s.last_pass == 3)
+	{
+	  ifstream i (s.translated_source.c_str());
+	  cout << i.rdbuf();
+	}
+
+      times (& tms_after);
+      gettimeofday (&tv_after, NULL);
+
+      if (s.verbose)
+	clog << _("Pass 3: translated to C into \"")
+	     << s.translated_source
+	     << "\" "
+	     << getmemusage()
+	     << TIMESPRINT
+	     << endl;
+
+      if (rc && ! s.try_server ())
+	cerr << _("Pass 3: translation failed.  [man error::pass3]") << endl;
+
+      PROBE1(stap, pass3__end, &s);
+
+      assert_no_interrupts();
+      if (rc || s.last_pass == 3)
+	return rc;
     }
 
-  times (& tms_after);
-  gettimeofday (&tv_after, NULL);
-
-  if (s.verbose) 
-    clog << _("Pass 3: translated to C into \"")
-         << s.translated_source
-         << "\" "
-         << getmemusage()
-         << TIMESPRINT
-         << endl;
-
-  if (rc && ! s.try_server ())
-    cerr << _("Pass 3: translation failed.  [man error::pass3]") << endl;
-
-  PROBE1(stap, pass3__end, &s);
-
-  assert_no_interrupts();
-  if (rc || s.last_pass == 3) return rc;
-
   // PASS 4: COMPILATION
   s.verbose = s.perpass_verbose[3];
   times (& tms_before);
   gettimeofday (&tv_before, NULL);
   PROBE1(stap, pass4__start, &s);
 
-  if (s.use_cache)
+  if (s.runtime_mode == systemtap_session::bpf_runtime)
+    rc = translate_bpf_pass (s);
+  else
     {
-      find_stapconf_hash(s);
-      get_stapconf_from_cache(s);
+      if (s.use_cache)
+	{
+	  find_stapconf_hash(s);
+	  get_stapconf_from_cache(s);
+	}
+      rc = compile_pass (s);
     }
-  rc = compile_pass (s);
-  if (! rc && s.last_pass == 4)
+
+  if (! rc && s.last_pass <= 4)
     {
       cout << ((s.hash_path == "") ? s.module_filename() : s.hash_path);
       cout << endl;
@@ -1066,18 +1142,21 @@ passes_0_4 (systemtap_session &s)
   times (& tms_after);
   gettimeofday (&tv_after, NULL);
 
-  if (s.verbose) clog << _("Pass 4: compiled C into \"")
-                      << s.module_filename()
-                      << "\" "
-                      << TIMESPRINT
-                      << endl;
+  if (s.verbose)
+    {
+      if (s.runtime_mode == systemtap_session::bpf_runtime)
+	clog << _("Pass 4: compiled BPF into \"");
+      else
+	clog << _("Pass 4: compiled C into \"");
+      clog << s.module_filename() << "\" " << TIMESPRINT << endl;
+    }
 
   if (rc && ! s.try_server ())
     cerr << _("Pass 4: compilation failed.  [man error::pass4]") << endl;
-
   else
     {
-      // Update cache. Cache cleaning is kicked off at the beginning of this function.
+      // Update cache. Cache cleaning is kicked off at the
+      // beginning of this function.
       if (s.use_script_cache)
         add_script_to_cache(s);
       if (s.use_cache && !s.runtime_usermode_p())
@@ -1085,7 +1164,7 @@ passes_0_4 (systemtap_session &s)
 
       // We may need to save the module in $CWD if the cache was
       // inaccessible for some reason.
-      if (! s.use_script_cache && s.last_pass == 4)
+      if (! s.use_script_cache && s.last_pass <= 4)
         s.save_module = true;
 
       // Copy module to the current directory.
@@ -1314,11 +1393,11 @@ main (int argc, char * const argv [])
 #if HAVE_NSS
 	    // If requested, query server status. This is independent
 	    // of other tasks.
-	    query_server_status (ss);
+	    nss_client_query_server_status (ss);
 
 	    // If requested, manage trust of servers. This is
 	    // independent of other tasks.
-	    manage_server_trust (ss);
+	    nss_client_manage_server_trust (ss);
 #endif
 
 	    // Run the passes only if a script has been specified or
