@@ -48,6 +48,14 @@
 #define dbug_trans2(args...) ;
 #endif
 
+#ifdef DEBUG_STP_ON_THE_FLY
+#define dbug_otf(args...) do {						\
+		_stp_dbug(__FUNCTION__, __LINE__, args);		\
+	} while (0)
+#else
+#define dbug_otf(args...) ;
+#endif
+
 #ifdef DEBUG_UNWIND /* stack unwinder */
 #define dbug_unwind(level, args...) do {					\
 		if ((level) <= DEBUG_UNWIND)				\
