@@ -154,6 +154,15 @@
 #define pt_regs_store_register(pt_regs,regno,value) \
   (pt_regs->gpr[regno] = (value))
 
+#elif defined __mips__
+
+#undef pt_regs_fetch_register
+#undef pt_regs_store_register
+#define pt_regs_fetch_register(pt_regs,regno) \
+  ((intptr_t) pt_regs->regs[regno])
+#define pt_regs_store_register(pt_regs,regno,value) \
+  (pt_regs->regs[regno] = (value))
+
 #elif defined (__aarch64__)
 
 #define pt_dwarf_register_0(pt_regs)	pt_regs->regs[0]
@@ -220,22 +229,19 @@
 #else
 #define DEREF_FAULT(addr) ({						    \
     snprintf(c->error_buffer, sizeof(c->error_buffer),			    \
-      STAP_MSG_LOC2C_01, (void *)(intptr_t)(addr), #addr);   \
+	     STAP_MSG_LOC2C_01, (long)(intptr_t)(addr));		    \
     c->last_error = c->error_buffer;					    \
     goto deref_fault;							    \
     })
 
 #define STORE_DEREF_FAULT(addr) ({					    \
     snprintf(c->error_buffer, sizeof(c->error_buffer),			    \
-      STAP_MSG_LOC2C_02, (void *)(intptr_t)(addr), #addr);  \
+	     STAP_MSG_LOC2C_02, (long)(intptr_t)(addr));		    \
     c->last_error = c->error_buffer;					    \
     goto deref_fault;							    \
     })
 
-#define CATCH_DEREF_FAULT()				\
-  if (0) {						\
-deref_fault: ;						\
-  }
+#define CATCH_DEREF_FAULT()	({0; }) /* always emitted by translator.cxx for functions & probes */
 #endif
 
 #endif /* _LOC2C_RUNTIME_H_ */
